(* Programa para calcular as Amplitudes de Helicidade para o 
   efeito de Compton com fermioes com massa.

 Last Version: 22/11/2021
 Author: Jorge C. Romao
 email: jorge.romao@tecnico.ulisboa.pt

*)

Remove["Global`*"]

(* Definitions *)
dp[s_]:= (1 + s DiracMatrix[5])/2

U[p_,s_]:= dp[s] . Spinor[p,0]

UBar[p_,s_]:=  SpinorUBar[p,0] . dp[-s]

PolSConjU = 
  Function[{k, p, l, i}, 
   If[i == 1, 2 U[k, l], If[i == 2, 2 U[p, -l]]]];
PolSConjUBar = 
  Function[{k, p, l, i}, 
   If[i == 1, UBar[p, l], If[i == 2, UBar[k, -l]]]];
PolSU = Function[{k, p, l, i}, 
   If[i == 1, 2 U[p, l], If[i == 2, 2 U[k, -l]]]];
PolSUBar = 
  Function[{k, p, l, i}, 
   If[i == 1, UBar[k, l], If[i == 2, UBar[p, -l]]]];
MydsU = Function[{p, i}, If[i == 1, U[p, 1], If[i == 2, U[p, -1]]]];
MydsUBar = 
  Function[{p, i}, If[i == 1, UBar[p, 1], If[i == 2, UBar[p, -1]]]];


  
(* Fermions with masss *)

Um[p1_,p2_,m_,s_]:=(If[s==1,MySP[p1,p2],MySPc[p2,p1]]/m U[p1,s] + U[p2,-s])
Vm[p1_,p2_,m_,s_]:=(If[s==1,MySP[p1,p2],MySPc[p2,p1]]/m U[p1,s] - U[p2,-s])

UmBar[p1_,p2_,m_,s_]:=(If[s==1,MySPc[p1,p2],MySP[p2,p1]]/m UBar[p1,s] 
		       + UBar[p2,-s])
VmBar[p1_,p2_,m_,s_]:=(If[s==1,MySPc[p1,p2],MySP[p2,p1]]/m UBar[p1,s] 
		       - UBar[p2,-s])

(* Amplitudes *)


MDiag1[s1_, s2_, l1_, l2_] := 
  Sum[(UmBar[w1, w2, m, s2].PolSConjU[k2, r1, l2, i1] PolSConjUBar[k2,
          r1, l2, i1].MydsU[r1, i2] MydsUBar[r1, i2] .PolSU[k1, r1, 
         l1, i3] PolSUBar[k1, r1, l1, i3].Um[r1, r2, m, s1] +
      UmBar[w1, w2, m, s2].PolSConjU[k2, r1, l2, i1] PolSConjUBar[k2, 
         r1, l2, i1].MydsU[r2, i2] MydsUBar[r2, i2] .PolSU[k1, r1, l1,
          i3] PolSUBar[k1, r1, l1, i3].Um[r1, r2, m, s1] +
      UmBar[w1, w2, m, s2].PolSConjU[k2, r1, l2, i1] PolSConjUBar[k2, 
         r1, l2, i1].MydsU[k1, i2] MydsUBar[k1, i2] .PolSU[k1, r1, l1,
          i3] PolSUBar[k1, r1, l1, i3].Um[r1, r2, m, s1]),
      {i1, 1, 2}, {i2, 1, 2}, {i3, 1, 2}] + m (
     Sum[UmBar[w1, w2, m, s2].PolSConjU[k2, r1, l2, i1] PolSConjUBar[k2, 
         r1, l2, i1].PolSU[k1, r1, l1, i2] PolSUBar[k1, r1, l1, 
         i2].Um[r1, r2, m, s1], {i1, 1, 2}, {i2, 1, 1}]);

MDiag2[s1_, s2_, l1_, l2_] := 
  Sum[(UmBar[w1, w2, m, s2].PolSU[k1, r1, l1, i1] PolSUBar[k1, r1, l1,
          i1].MydsU[r1, i2] MydsUBar[r1, i2] .PolSConjU[k2, r1, l2, 
         i3] PolSConjUBar[k2, r1, l2, i3].Um[r1, r2, m, s1] +
      UmBar[w1, w2, m, s2].PolSU[k1, r1, l1, i1] PolSUBar[k1, r1, l1, 
         i1].MydsU[r2, i2] MydsUBar[r2, i2] .PolSConjU[k2, r1, l2, 
         i3] PolSConjUBar[k2, r1, l2, i3].Um[r1, r2, m, s1] - 
      UmBar[w1, w2, m, s2].PolSU[k1, r1, l1, i1] PolSUBar[k1, r1, l1, 
         i1].MydsU[k2, i2] MydsUBar[k2, i2] .PolSConjU[k2, r1, l2, 
         i3] PolSConjUBar[k2, r1, l2, i3].Um[r1, r2, m, s1]),
    {i1, 1, 2}, {i2, 1, 2}, {i3, 1, 2}] + m (
     Sum[ 
      UmBar[w1, w2, m, s2].PolSU[k1, r1, l1, i1] PolSUBar[k1, r1, l1, 
         i1].PolSConjU[k2, r1, l2, i2] PolSConjUBar[k2, r1, l2, 
         i2].Um[r1, r2, m, s1], {i1, 1, 2}, {i2, 1, 2}]);

res1[s1_,s2_,l1_,l2_]:=DiracSimplify[DotSimplify[MDiag1[s1,s2,l1,l2]]]
res2[s1_,s2_,l1_,l2_]:=DiracSimplify[DotSimplify[MDiag2[s1,s2,l1,l2]]]

vlist={w1,w2,k1,k2,r1,r2}

simp1=Table[Spinor[vlist[[i]],0] . Spinor[vlist[[j]],0] -> MySP[vlist[[i]],vlist[[j]]] + MySPc[vlist[[j]],vlist[[i]]],{i,1,6},{j,1,6}] /. {MySP[p_, p_] -> 0, MySPc[q_, q_] -> 0}

simp2=Table[Spinor[vlist[[i]],0] . DiracMatrix[5] . Spinor[vlist[[j]],0] -> -MySP[vlist[[i]],vlist[[j]]] + MySPc[vlist[[j]],vlist[[i]]],{i,1,6},{j,1,6}] /. {MySP[p_, p_] -> 0, MySPc[q_, q_] -> 0}



simp=Flatten[{simp1,simp2}];


M1aux[s1_,s2_,l1_,l2_]:=Expand[res1[s1,s2,l1,l2] /. simp]
M2aux[s1_,s2_,l1_,l2_]:=Expand[res2[s1,s2,l1,l2] /. simp]

M1[s1_,s2_,l1_,l2_]:=Expand[M1aux[s1,s2,l1,l2] /. {MySP[p_, q_] -> sp[p, q], MySPc[p_, q_] -> spc[p, q]}]

M2[s1_,s2_,l1_,l2_]:=Expand[M2aux[s1,s2,l1,l2] /. {MySP[p_, q_] -> sp[p, q], MySPc[p_, q_] -> spc[p, q]}]


(* Change to True to write OutputFortran *)

If[False,

M={C1 M1[1,1,1,1] + C2 M2[1,1,1,1], \
C1 M1[1,1,1,-1] + C2 M2[1,1,1,-1], \
C1 M1[1,1,-1,1] + C2 M2[1,1,-1,1],\
C1 M1[1,1,-1,-1] + C2 M2[1,1,-1,-1], \
C1 M1[1,-1,1,1] + C2 M2[1,-1,1,1], \
C1 M1[1,-1,1,-1] + C2 M2[1,-1,1,-1], \
C1 M1[1,-1,-1,1] + C2 M2[1,-1,-1,1], \
C1 M1[1,-1,-1,-1] + C2 M2[1,-1,-1,-1], \
C1 M1[-1,1,1,1] + C2 M2[-1,1,1,1], \
C1 M1[-1,1,1,-1] + C2 M2[-1,1,1,-1], \
C1 M1[-1,1,-1,1] + C2 M2[-1,1,-1,1], \
C1 M1[-1,1,-1,-1] + C2 M2[-1,1,-1,-1], \
C1 M1[-1,-1,1,1] + C2 M2[-1,-1,1,1], \
C1 M1[-1,-1,1,-1] + C2 M2[-1,-1,1,-1], \
C1 M1[-1,-1,-1,1] + C2 M2[-1,-1,-1,1], \
C1 M1[-1,-1,-1,-1] + C2 M2[-1,-1,-1,-1]};


stmp=OpenWrite["ComptonAmplitudes.f",FormatType -> FortranForm,PageWidth -> 60];
Do[Write[stmp,"HelAmp(",i,")=",M[[i]]],{i,1,16}];
Close[stmp];

]
